local lib = LibStub:NewLibrary("LibFrameFade-1.0", "1")

-- Alpha animation stuff
FADEFRAMES = {};
local FlashFrames = {};


local FrameFlashTimers = {};
local FrameFlashTimerRefCount = {};
 
-- Function to see if a frame is already flashing
local function FrameIsFlashing(frame)
    for index, value in pairs(FlashFrames) do
        if ( value == frame ) then
            return 1;
        end
    end
    return nil;
end
 
-- Function to stop flashing
local function FrameFlashStop(frame)
    tDeleteItem(FlashFrames, frame);
    frame:SetAlpha(1.0);
    frame.flashTimer = nil;

    if ( frame.showWhenDone ) then
        frame:Show();
    else
        frame:Hide();
    end
end

-- Called every frame to update flashing frames
local function OMFrameFlash_OnUpdate(self, elapsed)
    local frame;
    local index = #FlashFrames;

    while FlashFrames[index] do
        frame = FlashFrames[index];
        frame.flashTimer = frame.flashTimer + elapsed;
        if ( (frame.flashTimer > frame.flashDuration) and frame.flashDuration ~= -1 ) then
            FrameFlashStop(frame);
        else
            local flashTime = frame.flashTimer;
            local alpha;
             
            flashTime = flashTime%(frame.fadeInTime+frame.fadeOutTime+(frame.flashInHoldTime or 0)+(frame.flashOutHoldTime or 0));
            if (flashTime < frame.fadeInTime) then
                alpha = flashTime/frame.fadeInTime;
            elseif (flashTime < frame.fadeInTime+(frame.flashInHoldTime or 0)) then
                alpha = 1;
            elseif (flashTime < frame.fadeInTime+(frame.flashInHoldTime or 0)+frame.fadeOutTime) then
                alpha = 1 - ((flashTime - frame.fadeInTime - (frame.flashInHoldTime or 0))/frame.fadeOutTime);
            else
                alpha = 0;
            end
             
            frame:SetAlpha(alpha);
            frame:Show();
        end
         
        -- Loop in reverse so that removing frames is safe
        index = index - 1;
    end
     
    if ( #FlashFrames == 0 ) then
        self:SetScript("OnUpdate", nil);
    end
end

local frameFlashManager = CreateFrame("FRAME");
 frameFlashManager:Show()

 -- Function to start a frame flashing
local function FrameFlash(frame, fadeInTime, fadeOutTime, flashDuration, showWhenDone, flashInHoldTime, flashOutHoldTime, syncId)
    if ( frame ) then
        local index = 1;
        -- If frame is already set to flash then return
        while FlashFrames[index] do
            if ( FlashFrames[index] == frame ) then
                return;
            end
            index = index + 1;
        end
         
        -- Time it takes to fade in a flashing frame
        frame.fadeInTime = fadeInTime;
        -- Time it takes to fade out a flashing frame
        frame.fadeOutTime = fadeOutTime;
        -- How long to keep the frame flashing
        frame.flashDuration = flashDuration;
        -- Show the flashing frame when the fadeOutTime has passed
        frame.showWhenDone = showWhenDone;
        -- Internal timer
        frame.flashTimer = 0;
        -- How long to hold the faded in state
        frame.flashInHoldTime = flashInHoldTime;
        -- How long to hold the faded out state
        frame.flashOutHoldTime = flashOutHoldTime;
         
        tinsert(FlashFrames, frame);
        frameFlashManager:SetScript("OnUpdate", OMFrameFlash_OnUpdate);
    end
end

lib.tables = tables
lib.ffades = ffades
lib.fflashs = fflashs
lib.frame = frame

lib.FrameFadeIn = FrameFadeIn
lib.FrameFadeOut = FrameFadeOut
lib.FrameFade = FrameFade
lib.FrameFadeRemoveFrame = faderelease
lib.FrameFlash = FrameFlash
lib.FrameFlashStop = FrameFlashStop
